#include <WiFi.h>
#include <WiFiUdp.h>
#include <esp_now.h>
#include <esp_wifi_internal.h>

#include <Ticker.h>
#include <time.h>
#include <driver/adc.h>
#include <driver/gpio.h>
#include <driver/i2s.h>
#include <esp_err.h>


#define   LED_PIN         22
#define   PDW_PIN        17
#define   FSY_PIN         4

const int Frame_Size = 192;

/*************************** Timer OS Code ************************************/
Ticker  mTickerTx;
Ticker  mTickerLed;

/*************************** Wifi Variables ************************************/
const char *ssid  = "CueSound";
const char *pass  = "head1234";

IPAddress ServerIP(10, 0, 1, 11);
IPAddress ClientIP(10, 0, 1, 12);


// Global copy of slave
esp_now_peer_info_t slave;

boolean       connected = false;

typedef
  struct  { 
   uint8_t PosTag;
   uint8_t PosTag2; 
   uint8_t PosTag3;
   uint8_t PosTag4;  
  }
TBuffer;

typedef union {
  struct  { 
   uint8_t PosTag;
   uint8_t PosTag2; 
   uint8_t PosTag3;
   uint8_t PosTag4;  
   int16_t samples[Frame_Size / 2];
  };
  uint8_t Data[Frame_Size+4];
}TBufferEntry;

TBufferEntry  DATA;
  
const int     RB_Size  = 32;
TBuffer       RingBuffer[RB_Size];    // buffer for incoming packets
volatile uint8_t       RingBegin  = 0;
volatile uint8_t       RingEnd  = 0;
volatile int8_t        RingSize  = 0;

uint16_t SampleLR[4];
int16_t  Buffer[16];
int      BufferCount = 0;
uint8_t  PacketCount  = 0;
volatile bool sent = true;
volatile bool CB = false;

uint8_t       LastPos;
volatile int8_t  PCount;

esp_err_t err;

/*************************** Sketch Code ************************************/
void setup() {
  Serial.begin(115200);

  RingBeginB  = 0;
  RingEndB  = 0;
  RingSizeB  = 0;

  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, !digitalRead(LED_PIN)); 
  pinMode(PDW_PIN, OUTPUT); 
  pinMode(FSY_PIN, OUTPUT);  

  delay(5000);
  Serial.println((String)"SERVER - BOOT");
  Serial.println((String)"SETUP ACCESS POINT");

  mTickerLed.attach_ms(1000, led_isr);

  EspNowSetup();
  delay(1000);
  Serial.println("SETUP I2S");
  i2sInit();    
  Serial.println("START I2S"); 
  size_t bytes_read = 1;
  uint32_t sample;
  while (bytes_read)  
    i2s_read(I2S_NUM_0, &Buffer, sizeof(Buffer), &bytes_read, 0); 
  Serial.println("START reader");
//  delay(3000);
  xTaskCreatePinnedToCore(reader, "ADC_reader", 2048, NULL, 1, NULL, 1);
}


void led_isr() {
  digitalWrite(LED_PIN, !digitalRead(LED_PIN));
  Serial.print(RingBegin); Serial.print(" / ");
  Serial.print(RingEnd); Serial.print(" / ");
  Serial.print(RingSize); Serial.print(" / ");
  Serial.println(PCount); 
}


void reader(void *pvParameters) {
  size_t bytes_read;
  int j = 0;

  while(1){ 
    for (j = 0; j < (Frame_Size / 2);) {
      err = i2s_read(I2S_NUM_0, &Buffer, sizeof(Buffer), &bytes_read, portMAX_DELAY);
 
      if (bytes_read == sizeof(Buffer)) {
        for (int i = 0; i < 16; i+=2) {
          DATA.samples[j] = Buffer[i+1];
          j++; 
        }
       } else {
        Serial.println("buffer empty"); 
      } 
    }
    
    PacketCount++;     
    RingBuffer[RingEnd].PosTag = PacketCount;
    RingBuffer[RingEnd].PosTag2 = PacketCount;
    RingBuffer[RingEnd].PosTag3 = PacketCount;
    RingBuffer[RingEnd].PosTag4 = 222;
        
    RingEnd = (RingEnd + 1) & (RB_Size-1); 
    noInterrupts();
    if (RingSize > (RB_Size-2)) { 
      Serial.println(" > 30 "); 
      RingBegin = (RingBegin + 1) & (RB_Size-1);  
     }
     else {
      RingSize++;
      PCount++;
    }
    interrupts();
    
    if (sent == true)  {
      sendData(); 
    }     
  }
}

void loop() {
 // rx();
//  delay(0);
}
