/*
 * This is an example to read analog data at high frequency using the I2S peripheral
 * Run a wire between pins 27 & 32
 * The readings from the device will be 12bit (0-4096) 
 */

#include "soc/rtc.h"
#include "soc/i2s_reg.h"

#define I2S_SAMPLE_RATE 44100
#define OUTPUT_PIN 27
#define OUTPUT_VALUE 3800


void i2sInit()
{
   i2s_config_t i2s_config = {
    .mode = (i2s_mode_t)(I2S_MODE_MASTER | I2S_MODE_RX ),
    .sample_rate =  I2S_SAMPLE_RATE,              
    .bits_per_sample = I2S_BITS_PER_SAMPLE_32BIT, 
    .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT,
    .communication_format = I2S_COMM_FORMAT_I2S,
    .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1,
    .dma_buf_count =  10,
    .dma_buf_len = Frame_Size / 4,
    .use_apll = true,
    .tx_desc_auto_clear = false,
//    .fixed_mclk = 0
   };
   err = i2s_driver_install(I2S_NUM_0, &i2s_config, 0, NULL);
   Serial.println(esp_err_to_name(err)); 
   static const i2s_pin_config_t  pin_config = {
    .bck_io_num = 2,
    .ws_io_num = 16,
    .data_out_num = I2S_PIN_NO_CHANGE,
    .data_in_num = 15
   };
   PIN_FUNC_SELECT(PERIPHS_IO_MUX_GPIO0_U, FUNC_GPIO0_CLK_OUT1);
   REG_WRITE(PIN_CTRL, 0);
   // PIN_FUNC_SELECT(PERIPHS_IO_MUX_U0RXD_U, FUNC_U0RXD_CLK_OUT2);
   // REG_WRITE(PIN_CTRL, 0xFFFFFF00);
   err = i2s_set_pin(I2S_NUM_0, &pin_config);
   Serial.println(esp_err_to_name(err));
   rtc_clk_apll_enable(1, 129, 8, 5, 2);
    Serial.println(READ_PERI_REG(I2S_SAMPLE_RATE_CONF_REG(0))); 
    Serial.println(READ_PERI_REG(I2S_CLKM_CONF_REG(0))); 
   digitalWrite(PDW_PIN, HIGH);  
   digitalWrite(FSY_PIN, HIGH);
}
